@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023,2024 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -23,9 +23,7 @@
 #include <mobius/exception.inc>
 #include <stdexcept>
 
-namespace mobius
-{
-namespace py
+namespace mobius::py
 {
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get function argument list size
@@ -169,10 +167,50 @@ get_arg_as_bool (PyObject *args, std::uint32_t idx, bool value)
 {
   if (idx < get_arg_size (args))
     return pybool_as_bool (get_arg (args, idx));
-  
+
   return value;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get function argument as char
+//! \param args Argument tuple
+//! \param idx Argument index
+//! \return Argument
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+char
+get_arg_as_char (PyObject *args, std::uint32_t idx)
+{
+  Py_ssize_t siz = 0;
+  const char *buffer = PyUnicode_AsUTF8AndSize (get_arg (args, idx), &siz);
+
+  if (buffer)
+    {
+      if (siz != 1)
+        throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("invalid char"));
+
+      return buffer[0];
+    }
+
+  else
+    throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("invalid Python string"));
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get function argument as char
+//! \param args Argument tuple
+//! \param idx Argument index
+//! \param c Default value
+//! \return Argument
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+char
+get_arg_as_char (PyObject *args, std::uint32_t idx, char c)
+{
+  if (idx < get_arg_size (args))
+    return get_arg_as_char (args, idx);
+
+  return c;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get function argument as int
 //! \param args Argument tuple
@@ -197,7 +235,7 @@ get_arg_as_int (PyObject *args, std::uint32_t idx, int value)
 {
   if (idx < get_arg_size (args))
     return PyLong_AsLong (get_arg (args, idx));
-  
+
   return value;
 }
 
@@ -225,7 +263,7 @@ get_arg_as_int32_t (PyObject *args, std::uint32_t idx, std::int32_t value)
 {
   if (idx < get_arg_size (args))
     return pylong_as_std_int32_t (get_arg (args, idx));
-  
+
   return value;
 }
 
@@ -253,7 +291,7 @@ get_arg_as_int64_t (PyObject *args, std::uint32_t idx, std::int64_t value)
 {
   if (idx < get_arg_size (args))
     return pylong_as_std_int64_t (get_arg (args, idx));
-  
+
   return value;
 }
 
@@ -281,7 +319,7 @@ get_arg_as_uint8_t (PyObject *args, std::uint32_t idx, std::uint8_t value)
 {
   if (idx < get_arg_size (args))
     return get_arg_as_uint8_t (args, idx);
-  
+
   return value;
 }
 
@@ -309,7 +347,7 @@ get_arg_as_uint16_t (PyObject *args, std::uint32_t idx, std::uint16_t value)
 {
   if (idx < get_arg_size (args))
     return get_arg_as_uint16_t (args, idx);
-  
+
   return value;
 }
 
@@ -337,7 +375,7 @@ get_arg_as_uint32_t (PyObject *args, std::uint32_t idx, std::uint32_t value)
 {
   if (idx < get_arg_size (args))
     return get_arg_as_uint32_t (args, idx);
-  
+
   return value;
 }
 
@@ -365,7 +403,7 @@ get_arg_as_uint64_t (PyObject *args, std::uint32_t idx, std::uint64_t value)
 {
   if (idx < get_arg_size (args))
     return get_arg_as_uint64_t (args, idx);
-  
+
   return value;
 }
 
@@ -437,6 +475,36 @@ set_stop_iteration ()
   PyErr_SetNone (PyExc_StopIteration);
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get error message
+//! \return Error message
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::string
+get_error_message ()
+{
+  PyObject *exc_type, *exc_value, *exc_traceback;
+  PyErr_Fetch (&exc_type, &exc_value, &exc_traceback);
+  PyErr_NormalizeException (&exc_type, &exc_value, &exc_traceback);
+
+  std::string msg = mobius::py::pystring_as_std_string (PyObject_Str (exc_type))
+    + ' ' + mobius::py::pystring_as_std_string (PyObject_Str (exc_value));
+
+  auto p_traceback = reinterpret_cast <PyTracebackObject *> (exc_traceback);
+
+  while (p_traceback)
+    {
+      msg += "\nFile: \"" + mobius::py::pystring_as_std_string (PyObject_Str (p_traceback->tb_frame->f_code->co_filename)) + "\", line " + std::to_string (p_traceback->tb_lineno);
+
+      p_traceback = p_traceback->tb_next;
+    }
+
+  Py_XDECREF (exc_type);
+  Py_XDECREF (exc_value);
+  Py_XDECREF (exc_traceback);
+
+  return msg;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get current line number
 //! \return Line number
@@ -450,7 +518,7 @@ get_line_number ()
 
   if (frame)
     line = PyFrame_GetLineNumber (frame);
-  
+
   return line;
 }
 
@@ -462,11 +530,11 @@ std::string
 get_filename ()
 {
   std::string filename;
-  
+
   PyFrameObject *frame = PyEval_GetFrame ();
   if (frame && frame->f_code && frame->f_code->co_filename)
     filename = mobius::py::pystring_as_std_string (frame->f_code->co_filename);
-  
+
   return filename;
 }
 
@@ -478,23 +546,12 @@ std::string
 get_funcname ()
 {
   std::string funcname;
-  
+
   PyFrameObject *frame = PyEval_GetFrame ();
   if (frame && frame->f_code && frame->f_code->co_name)
     funcname = mobius::py::pystring_as_std_string (frame->f_code->co_name);
-  
-  return funcname;
-}
 
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Check if object is Py_None
-//! \param value Python object
-//! \return true/false
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-bool
-pynone_check (PyObject *value)
-{
-  return value == Py_None;
+  return funcname;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -508,6 +565,50 @@ pynone ()
   return Py_None;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create new instance of Py_True
+//! \return Python object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+PyObject *
+py_true ()
+{
+  Py_INCREF (Py_True);
+  return Py_True;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create new instance of Py_False
+//! \return Python object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+PyObject *
+py_false ()
+{
+  Py_INCREF (Py_False);
+  return Py_False;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create new instance of Py_NotImplemented
+//! \return Python object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+PyObject *
+py_not_implemented ()
+{
+  Py_INCREF (Py_NotImplemented);
+  return Py_NotImplemented;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Check if object is Py_None
+//! \param value Python object
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+pynone_check (PyObject *value)
+{
+  return value == Py_None;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Check string
 //! \param value Python object
@@ -527,7 +628,12 @@ pystring_check (PyObject *value)
 PyObject *
 pystring_from_std_string (const std::string& s)
 {
-  return PyUnicode_FromString (s.c_str ());
+  auto ret = PyUnicode_DecodeUTF8 (s.c_str (), s.size (), "replace");
+
+  if (!ret)
+    throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("invalid C++ UTF-8 string"));
+
+  return ret;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -545,7 +651,7 @@ pystring_as_std_string (PyObject *value)
     str = buffer;
 
   else
-    throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("Invalid Python string"));
+    throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("invalid Python string"));
 
   return str;
 }
@@ -622,7 +728,7 @@ pybool_from_bool (bool b)
 {
   PyObject *ret = b ? Py_True : Py_False;
   Py_INCREF (ret);
-  
+
   return ret;
 }
 
@@ -821,7 +927,12 @@ pylong_from_std_uint64_t (std::uint64_t value)
 std::uint64_t
 pylong_as_std_uint64_t (PyObject *value)
 {
-  return PyLong_AsUnsignedLongLong (value);
+  std::uint64_t v = PyLong_AsUnsignedLongLong (value);
+
+  if (PyErr_Occurred ())
+    throw std::runtime_error (get_error_message ());
+
+  return v;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -879,5 +990,4 @@ pyfloat_from_cpp (double value)
   return PyFloat_FromDouble (value);
 }
 
-} // namespace py
-} // namespace mobius
+} // namespace mobius::py
