@@ -40,6 +40,7 @@ std::string get_arg_as_std_string (PyObject *, std::uint32_t);
 mobius::bytearray get_arg_as_bytearray (PyObject *, std::uint32_t);
 mobius::datetime::datetime get_arg_as_datetime (PyObject *, std::uint32_t);
 bool get_arg_as_bool (PyObject *, std::uint32_t);
+char get_arg_as_char (PyObject *, std::uint32_t);
 int get_arg_as_int (PyObject *, std::uint32_t);
 std::int32_t get_arg_as_int32_t (PyObject *, std::uint32_t);
 std::int64_t get_arg_as_int64_t (PyObject *, std::uint32_t);
@@ -55,6 +56,7 @@ std::string get_arg_as_std_string (PyObject *, std::uint32_t, const std::string&
 mobius::bytearray get_arg_as_bytearray (PyObject *, std::uint32_t, const mobius::bytearray&);
 mobius::datetime::datetime get_arg_as_datetime (PyObject *, std::uint32_t, const mobius::datetime::datetime&);
 bool get_arg_as_bool (PyObject *, std::uint32_t, bool);
+char get_arg_as_char (PyObject *, std::uint32_t, char);
 int get_arg_as_int (PyObject *, std::uint32_t, int);
 std::int32_t get_arg_as_int32_t (PyObject *, std::uint32_t, std::int32_t);
 std::int64_t get_arg_as_int64_t (PyObject *, std::uint32_t, std::int64_t);
@@ -280,84 +282,6 @@ pytuple_from_cpp_pair (const T& p, F1 pyf1, F2 pyf2)
   return ret;
 }
 
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Create new Python dict from C++ container
-//! \param container C++ container
-//! \param pyfk Function to convert C++ keys to Python objects
-//! \param pyfv Function to convert C++ values to Python objects
-//! \return Python dict
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-template <typename C, typename FK, typename FV> PyObject *
-pydict_from_cpp_container (const C& container, FK pyfk, FV pyfv)
-{
-  PyObject *ret = PyDict_New ();
-
-  if (!ret)
-    return nullptr;
-
-  for (const auto& p : container)
-    {
-      PyObject *py_key = pyfk (p.first);
-
-      if (!py_key)
-        {
-          Py_CLEAR (ret);
-          return nullptr;
-        }
-
-      PyObject *py_value = pyfv (p.second);
-
-      if (!py_value)
-        {
-          Py_DECREF (py_key);
-          Py_CLEAR (ret);
-          return nullptr;
-        }
-
-      PyDict_SetItem (ret, py_key, py_value);
-      Py_DECREF (py_key);
-      Py_DECREF (py_value);
-    }
-
-  return ret;
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Create new Python set from C++ set
-//! \param container C++ set
-//! \param pyfunc Function to convert C++ items to Python objects
-//! \return Python set object
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-template <typename C, typename F> PyObject *
-pyset_from_cpp_container (const C& container, F pyfunc)
-{
-  PyObject *ret = PySet_New (NULL);
-
-  if (!ret)
-    return nullptr;
-
-  for (const auto& item : container)
-    {
-      PyObject *py_item = pyfunc (item);
-
-      if (!py_item)
-        {
-          Py_CLEAR (ret);
-          return nullptr;
-        }
-
-      if (PySet_Add (ret, py_item) == -1)
-        {
-          Py_CLEAR (ret);
-          return nullptr;
-        }
-
-      Py_DECREF (py_item);
-    }
-
-  return ret;
-}
-
 } // namespace mobius::py
 
 #endif
