@@ -18,7 +18,6 @@
 import os.path
 
 import mobius
-import pymobius.p2p.ares
 import pymobius.p2p.emule
 import pymobius.p2p.emuletorrent
 import pymobius.p2p.shareaza
@@ -161,8 +160,6 @@ class Ant(object):
             os.remove(path)
 
         # Retrieve activity data
-        mobius.core.logf('DBG 3.1')
-        pymobius.p2p.ares.retrieve(self.__model)
         mobius.core.logf('DBG 3.2')
         pymobius.p2p.emule.retrieve(self.__model)
         mobius.core.logf('DBG 3.3')
